(function webpackUniversalModuleDefinition(root, factory) {
	if(typeof exports === 'object' && typeof module === 'object')
		module.exports = factory();
	else if(typeof define === 'function' && define.amd)
		define([], factory);
	else if(typeof exports === 'object')
		exports["MarkdownEditor"] = factory();
	else
		root["MarkdownEditor"] = factory();
})(self, () => {
return /******/ (() => { // webpackBootstrap
/******/ 	"use strict";
var __webpack_exports__ = {};
// This entry need to be wrapped in an IIFE because it uses a non-standard name for the exports (exports).
(() => {
var exports = __webpack_exports__;

Object.defineProperty(exports, "__esModule", ({ value: true }));
class MarkdownEditor {
    constructor(box) {
        this.config = {
            enableAutoCompletion: false,
            bracketMatchPairs: [
                {
                    enabled: true,
                    left: '(',
                    right: ')',
                },
                {
                    enabled: true,
                    left: '[',
                    right: ']',
                },
                {
                    enabled: true,
                    left: '{',
                    right: '}',
                },
                {
                    enabled: true,
                    left: '``',
                    right: '`\n```',
                    backspace: '``',
                    moveBack: 4,
                    trigger: '`',
                    needLeftMatch: true,
                },
                {
                    enabled: true,
                    left: '`',
                    right: '`',
                },
                {
                    enabled: true,
                    left: '*',
                    right: '***',
                    moveBack: 2,
                    rollback: '*',
                    trigger: '*',
                    needLeftMatch: true,
                },
                {
                    enabled: true,
                    left: '_',
                    right: '___',
                    moveBack: 2,
                    rollback: '_',
                    trigger: '_',
                    needLeftMatch: true,
                },
                {
                    enabled: true,
                    left: '"',
                    right: '"',
                },
                {
                    enabled: false,
                    left: "'",
                    right: "'",
                },
                {
                    enabled: false,
                    left: '$$',
                    right: '\n\n$$',
                    backspace: '$$',
                    moveBack: 3,
                    trigger: '$',
                    needLeftMatch: true,
                },
                {
                    enabled: false,
                    left: '$',
                    right: '$',
                },
            ],
        };
        this.disabled = false;
        this.hotKeyMap = [
            {
                key: 'ctrl+`',
                action: (editor) => editor.code(),
            },
            {
                key: 'ctrl+b',
                action: (editor) => editor.bold(),
            },
            {
                key: 'ctrl+q',
                action: (editor) => editor.quote(),
            },
            {
                key: 'ctrl+k',
                action: (editor) => editor.link(),
            },
            {
                key: 'ctrl+shift+f',
                action: (editor) => editor.table(),
            },
            {
                key: 'shift+tab',
                action: (editor) => editor.decreaseIndent(),
            },
            {
                key: 'tab',
                action: (editor) => editor.increaseIndent(),
            },
        ];
        this.textbox = box;
    }
    focusEditor() {
        if (this.disabled)
            return;
        const editor = this.textbox;
        setTimeout(function () {
            editor.focus();
        }, 10);
    }
    getSelection() {
        const lo = this.textbox.selectionStart;
        const hi = this.textbox.selectionEnd;
        return this.textbox.value.substring(lo, hi);
    }
    hasSelection() {
        return this.textbox.selectionStart !== this.textbox.selectionEnd;
    }
    clearSelectionToStart() {
        if (!this.hasSelection()) {
            return;
        }
        this.textbox.selectionEnd = this.textbox.selectionStart;
    }
    clearSelectionToEnd() {
        if (!this.hasSelection()) {
            return;
        }
        this.textbox.selectionStart = this.textbox.selectionEnd;
    }
    replaceSelectionWith(replacement) {
        const diff = replacement.length - this.getSelection().length;
        const newStart = this.textbox.selectionStart;
        const newEnd = this.textbox.selectionEnd + diff;
        if (!document.execCommand('insertText', false, replacement)) {
            this.textbox.setRangeText(replacement);
        }
        this.textbox.selectionStart = newStart;
        this.textbox.selectionEnd = newEnd;
        this.focusEditor();
    }
    wrapSelectionWith(prefix, suffix, lineByline) {
        return this.wrapSelectionWithGenerator(() => prefix, () => suffix, lineByline);
    }
    wrapSelectionWithGenerator(prefixGenerator, suffixGenerator, lineByline) {
        const value = this.getSelection();
        if (value.length === 0) {
            this.insertToCaret(prefixGenerator(0) + suffixGenerator(0));
            return;
        }
        if (lineByline) {
            const lines = value.split('\n');
            let replacement = '';
            for (let i = 0; i < lines.length; i++) {
                if (lines[i].endsWith('\r')) {
                    lines[i] = lines[i].slice(0, -1);
                }
                if (lines[i].length > 0) {
                    replacement += prefixGenerator(i) + lines[i] + suffixGenerator(i);
                }
                if (i !== lines.length - 1) {
                    replacement += '\n';
                }
            }
            this.replaceSelectionWith(replacement);
        }
        else {
            this.replaceSelectionWith(prefixGenerator(0) + value + suffixGenerator(0));
        }
        this.focusEditor();
    }
    getCurrentLine() {
        const caretPosition = this.textbox.selectionStart;
        const lastLine = this.textbox.value.substring(0, caretPosition).lastIndexOf('\n');
        return this.textbox.value.substring(lastLine + 1, caretPosition);
    }
    getStringRelativeToCaret(offset) {
        if (offset < 0) {
            const start = this.textbox.selectionStart;
            return this.textbox.value.substring(start + offset, start);
        }
        const end = this.textbox.selectionEnd;
        return this.textbox.value.substring(end, end + offset);
    }
    insertToCaret(value) {
        if (!document.execCommand('insertText', false, value)) {
            const index = this.textbox.selectionEnd;
            this.textbox.setRangeText(value, index, index, 'end');
        }
        this.focusEditor();
    }
    moveCaretBy(count) {
        if (!this.hasSelection()) {
            if (count > 0) {
                this.textbox.selectionStart += count;
            }
            else {
                this.textbox.selectionEnd += count;
            }
        }
        else {
            this.textbox.selectionStart += count;
            this.textbox.selectionEnd += count;
        }
    }
    getDraftKey(app, userId, postId) {
        return app + '-draft-' + userId + '-' + postId;
    }
    saveDraft(app, userId, postId) {
        if (this.textbox.value.length > 4000) {
            return;
        }
        window.localStorage.setItem(this.getDraftKey(app, userId, postId), this.textbox.value);
    }
    loadDraft(app, userId, postId) {
        const value = window.localStorage.getItem(this.getDraftKey(app, userId, postId));
        if (value && !this.textbox.value) {
            this.textbox.value = value;
        }
    }
    removeDraft(app, userId, postId) {
        window.localStorage.removeItem(this.getDraftKey(app, userId, postId));
    }
    getConfigKey(app, userId) {
        return app + '-config-' + userId;
    }
    saveConfig(app, userId) {
        window.localStorage.setItem(this.getConfigKey(app, userId), JSON.stringify(this.config));
    }
    loadConfig(app, userId) {
        const value = window.localStorage.getItem(this.getConfigKey(app, userId));
        if (value) {
            this.config = Object.assign(Object.assign({}, this.config), JSON.parse(value));
        }
    }
    bracketMatch(input) {
        for (let i = 0; i < this.config.bracketMatchPairs.length; i++) {
            const current = this.config.bracketMatchPairs[i];
            if (current.enabled === false) {
                continue;
            }
            const leftMatched = this.getStringRelativeToCaret(-current.left.length) === current.left;
            const rightMatched = this.getStringRelativeToCaret(current.right.length) === current.right;
            if (input === current.right && rightMatched) {
                this.moveCaretBy(current.right.length);
                return true;
            }
            if (input === (current.trigger || current.left)) {
                if (this.hasSelection()) {
                    if (!current.needLeftMatch) {
                        this.wrapSelectionWith(current.left, current.right, false);
                        return true;
                    }
                    continue;
                }
                if (leftMatched && current.needLeftMatch) {
                    this.insertToCaret(current.right);
                    this.moveCaretBy(-(current.moveBack || current.right.length));
                    return true;
                }
                if (!current.needLeftMatch) {
                    this.insertToCaret(current.left + current.right);
                    this.moveCaretBy(-(current.moveBack || current.right.length));
                    return true;
                }
            }
        }
        return false;
    }
    bold() {
        this.wrapSelectionWith('**', '**', true);
        if (this.hasSelection()) {
            this.clearSelectionToEnd();
        }
        else {
            this.moveCaretBy(-2);
        }
    }
    link() {
        const selection = this.getSelection();
        if (selection) {
            try {
                new URL(selection);
                this.wrapSelectionWith('[](', ')', true);
                this.clearSelectionToStart();
                this.moveCaretBy(1);
                return;
            }
            catch (_a) {
            }
            this.wrapSelectionWith('[', ']()', true);
            this.clearSelectionToEnd();
            this.moveCaretBy(-1);
            return;
        }
        this.wrapSelectionWith('[', ']()', true);
        this.clearSelectionToEnd();
        this.moveCaretBy(-3);
    }
    quote() {
        this.wrapSelectionWith('> ', '', true);
        if (this.hasSelection()) {
            this.clearSelectionToEnd();
        }
    }
    code() {
        if (!this.hasSelection()) {
            this.insertToCaret('``');
            this.moveCaretBy(-1);
            return;
        }
        const selection = this.getSelection();
        const newlineIndex = selection.lastIndexOf('\n');
        if (newlineIndex > 0) {
            let suffix = '```\n';
            if (newlineIndex !== selection.length - 1) {
                suffix = '\n' + suffix;
            }
            this.wrapSelectionWith('```\n', suffix, false);
        }
        else {
            this.wrapSelectionWith('`', '`', false);
        }
        this.clearSelectionToEnd();
    }
    collapsibleCodeBlock() {
        let prefix = '<details>\n<summary>点击查看代码</summary>\n\n```\n';
        const suffix = '\n```\n</details>';
        if (!this.hasSelection()) {
            this.insertToCaret(prefix + suffix);
            this.moveCaretBy(-suffix.length);
            return;
        }
        const selectionStart = this.textbox.selectionStart;
        if (this.textbox.value[selectionStart - 1] !== '\n') {
            prefix = '\n' + prefix;
        }
        this.wrapSelectionWith(prefix, suffix, false);
        this.moveCaretBy(-suffix.length);
        this.clearSelectionToEnd();
    }
    taskLists() {
        let prefix = '- [ ] ';
        const suffix = '';
        if (!this.hasSelection()) {
            this.insertToCaret(prefix + suffix);
            this.moveCaretBy(-suffix.length);
            return;
        }
        const selectionStart = this.textbox.selectionStart;
        if (this.textbox.value[selectionStart - 1] !== '\n') {
            prefix = '\n' + prefix;
        }
        this.wrapSelectionWith(prefix, suffix, false);
        this.moveCaretBy(-suffix.length);
        this.clearSelectionToEnd();
    }
    table(col, row) {
        if (col == undefined && row == undefined) {
            if (this.hasSelection()) {
                const rows = this.getSelection()
                    .trimRight()
                    .split('\n')
                    .map(s => s.trim())
                    .map(s => (s.startsWith('|') ? s.slice(1) : s))
                    .map(s => (s.endsWith('|') ? s.slice(0, -1) : s))
                    .map(s => s.split('|'));
                this.buildTable(rows);
                return;
            }
            else {
                col = 2;
                row = 3;
            }
        }
        this.buildTable(new Array(col).fill('    ').map(c => new Array(row).fill(c)));
    }
    numberedList() {
        this.wrapSelectionWithGenerator(n => n + 1 + '. ', () => '', true);
    }
    bulletList() {
        this.wrapSelectionWith('* ', '', true);
    }
    buildTable(rows) {
        const hasTableHeaderSplitter = rows.length > 1 &&
            rows[1].every(i => i
                .trim()
                .split('')
                .every(c => c === '-' || c === ':'));
        if (hasTableHeaderSplitter) {
            rows.splice(1, 1);
        }
        rows = rows.map(r => r.map(col => col.trim()));
        rows = this.formatTable(rows);
        const col = hasTableHeaderSplitter ? rows[0].length : Math.max(...rows.map(r => r.length), 2);
        while (rows[0].length < col) {
            rows[0].push('header');
        }
        if (rows.length == 1) {
            rows.push(new Array(col).fill('    '));
        }
        rows = this.alignTable(rows);
        const headerLine = '|' + rows[0].join('|') + '|';
        const headerSplitter = '| ' + rows[0].map(s => new Array(s.length - 1).join('-')).join(' | ') + ' |';
        const content = rows
            .slice(1)
            .map(r => {
            r.length = col;
            return '|' + r.join('|') + '|';
        })
            .join('\n');
        this.replaceSelectionWith(headerLine + '\n' + headerSplitter + '\n' + content);
        this.clearSelectionToEnd();
        this.focusEditor();
    }
    formatTable(rows) {
        if (rows.length < 2) {
            return rows;
        }
        const placeholder = (num) => new Array(num === 0 ? 4 : num).fill(' ').join('');
        const diff = rows[0].length - Math.max(...rows.slice(1).map(r => r.length));
        if (diff > 0) {
            for (let i = 1; i < rows.length; i++) {
                const zippedRow = [];
                rows[0].forEach((cur, colIndex) => {
                    var _a;
                    return zippedRow.push(cur === ''
                        ? placeholder(rows[i - 1][colIndex].length)
                        : (_a = rows[i].shift()) !== null && _a !== void 0 ? _a : placeholder(rows[i - 1][colIndex].length));
                });
                rows[i] = zippedRow;
            }
            rows[0] = rows[0].map(cur => (cur === '' ? placeholder(0) : cur));
        }
        else {
            for (let i = 1; i < rows.length; i++) {
                let zippedRow = [];
                rows[0].forEach((cur, colIndex) => {
                    var _a;
                    let toAdd;
                    if (cur === '') {
                        if (rows[i].length > 0) {
                            rows[i].shift();
                            toAdd = '';
                        }
                        else {
                            toAdd = placeholder(rows[i - 1][colIndex].length);
                        }
                    }
                    else {
                        toAdd = (_a = rows[i].shift()) !== null && _a !== void 0 ? _a : placeholder(rows[i - 1][colIndex].length);
                        if (toAdd === '') {
                            toAdd = placeholder(rows[i - 1][colIndex].length);
                        }
                    }
                    zippedRow.push(toAdd);
                });
                rows[i] = zippedRow;
            }
            rows = rows.map(row => row.filter(s => s !== ''));
        }
        return rows;
    }
    alignTable(rows) {
        rows[0].forEach((cur, col) => {
            const max = Math.max(...rows.map(r => r[col].length), 4) + 2;
            rows.forEach(r => (r[col] = r[col].padEnd(max - 1).padStart(max)));
        });
        return rows;
    }
    increaseIndent() {
        if (this.hasSelection()) {
            this.wrapSelectionWith('  ', '', true);
        }
        else {
            this.insertToCaret('  ');
        }
    }
    decreaseIndent() {
        const lines = this.getSelection().split('\n');
        let trimString = '';
        for (let i = 0; i < lines.length; i++) {
            if (lines[i].startsWith('  ')) {
                lines[i] = lines[i].slice(2);
            }
            else if (lines[i].startsWith(' ')) {
                lines[i] = lines[i].slice(1);
            }
            if (lines[i].endsWith('\r')) {
                lines[i] = lines[i].slice(0, -1);
            }
            trimString += lines[i];
            if (i !== lines.length - 1) {
                trimString += '\n';
            }
        }
        this.replaceSelectionWith(trimString);
    }
    isPastingTable(data) {
        const typeSet = new Set(data.types);
        if (typeSet.has('text/html') && typeSet.has('text/plain')) {
            const doc = new DOMParser().parseFromString(data.getData('text/html'), 'text/html');
            if (doc.getElementsByTagName('table').length !== 1) {
                return false;
            }
            const text = data.getData('text/plain').trim();
            const rows = text.split(/[\n\u0085\u2028\u2029]|\r\n?/g);
            if (doc.querySelectorAll('tr').length !== rows.length) {
                return false;
            }
            const textMaxColumn = Math.max(...rows.map(r => r.split('\t').length));
            let htmlMaxColumn = 0;
            doc
                .querySelectorAll('table tr')
                .forEach((row) => (htmlMaxColumn = Math.max(htmlMaxColumn, row.cells.length)));
            return textMaxColumn === htmlMaxColumn;
        }
        return false;
    }
    isPastingCode(data) {
        const typeSet = new Set(data.types);
        if (typeSet.has('text/markdown')) {
            return false;
        }
        if (typeSet.has('text/html') && typeSet.has('text/plain')) {
            if (typeSet.has('vscode-editor-data')) {
                try {
                    const { mode } = JSON.parse(data.getData('vscode-editor-data'));
                    return mode !== 'markdown';
                }
                catch (e) {
                    return true;
                }
            }
            const doc = new DOMParser().parseFromString(data.getData('text/html'), 'text/html');
            return doc.getElementsByTagName('pre').length === 1;
        }
    }
    onKeydown(e) {
        if (e.isComposing || e.keyCode === 229) {
            return true;
        }
        for (let i = 0; i < this.hotKeyMap.length; i++) {
            let canApply = true;
            const keys = this.hotKeyMap[i].key.split('+');
            for (let j = 0; j < keys.length; j++) {
                if (keys[j].toLowerCase() === 'ctrl'.toLowerCase()) {
                    canApply = canApply && (e.ctrlKey || e.metaKey);
                }
                else if (keys[j].toLowerCase() === 'shift'.toLowerCase()) {
                    canApply = canApply && e.shiftKey;
                }
                else if (keys[j].toLowerCase() === 'alt'.toLowerCase()) {
                    canApply = canApply && e.altKey;
                }
                else {
                    canApply = canApply && e.key.toLowerCase() === keys[j].toLowerCase();
                }
            }
            if (canApply) {
                this.hotKeyMap[i].action(this);
                e.preventDefault();
                e.stopPropagation();
                return false;
            }
        }
        if (!this.config.enableAutoCompletion) {
            return true;
        }
        if (this.bracketMatch(e.key)) {
            e.preventDefault();
            e.stopPropagation();
            return false;
        }
        if (e.key === 'Enter') {
            const currentLine = this.getCurrentLine();
            const listMark = currentLine.substring(0, 2);
            if (listMark === '* ' || listMark === '- ') {
                this.insertToCaret('\n' + listMark);
                e.preventDefault();
                e.stopPropagation();
                return false;
            }
            const ol = currentLine.match(/^(\d+)([.)]) [^\n]*/);
            if (ol) {
                const nextIndex = parseInt(ol[1]) + 1;
                this.insertToCaret('\n' + nextIndex + ol[2] + ' ');
                e.preventDefault();
                e.stopPropagation();
                return false;
            }
        }
        return true;
    }
    onPaste(e) {
        const clipboardData = e.clipboardData;
        let preventDefault = false;
        try {
            if (clipboardData && clipboardData.items) {
                if (this.isPastingTable(clipboardData)) {
                    const rows = clipboardData
                        .getData('text/plain')
                        .trim()
                        .split(/[\n\u0085\u2028\u2029]|\r\n?/g)
                        .map(row => row.split('\t'));
                    this.buildTable(rows);
                    preventDefault = true;
                }
                else if (this.config.enableAutoCompletion && this.isPastingCode(clipboardData)) {
                    const code = clipboardData.getData('text/plain');
                    const editorData = clipboardData.getData('vscode-editor-data');
                    const { mode } = JSON.parse(editorData ? editorData : '{ "mode": "plaintext" }');
                    if (code.indexOf('\n') >= 0) {
                        this.insertToCaret('```' + mode ? mode : 0);
                        this.moveCaretBy(-4);
                    }
                    else {
                        this.insertToCaret('``');
                        this.moveCaretBy(-1);
                    }
                    preventDefault = false;
                }
            }
        }
        catch (_a) {
            preventDefault = false;
        }
        if (preventDefault) {
            e.preventDefault();
        }
    }
}
exports["default"] = MarkdownEditor;

})();

__webpack_exports__ = __webpack_exports__["default"];
/******/ 	return __webpack_exports__;
/******/ })()
;
});